import Flutter
import UIKit
import MapmyIndiaNearbyUI
import MapmyIndiaAPIKit
public class SwiftMapmyindiaNearbyPlugin: NSObject, FlutterPlugin {
    private var flutterViewController: FlutterViewController!
    var nearbyUI: MapmyIndiaNearbyCategoriesViewController!
    var result : FlutterResult!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "mapmyindia_nearby_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftMapmyindiaNearbyPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        flutterViewController = (UIApplication.shared.delegate?.window??.rootViewController) as? FlutterViewController
        if call.method == "openNearbyWidget" {
            let config = MapmyIndiaNearbyFlutterConfiguration()
            nearbyUI = MapmyIndiaNearbyCategoriesViewController()
            nearbyUI.mapmyIndiaNearbyConfiguration.mapmyIndiaNearbyUIConfiguration.nearbyPresentStyle = .fullScreen
            nearbyUI.modalPresentationStyle = .fullScreen
            nearbyUI.delegate = self
            
            var argsLists: [String: Any]
            if let args = call.arguments {
                argsLists = (args as? [String : Any] ?? [String: Any]())
                config.setNearbyConfiguration(argsLists: argsLists, nearbyPlugin: nearbyUI)
            }
//            nearbyUI.mapmyIndiaNearbyConfiguration.mapmyIndiaNearbyFilterConfiguration.pod = .district
            flutterViewController.present(nearbyUI, animated: false, completion: nil)
            self.result = result
        }
    }
    
    func sendCallback(place:MapmyIndiaAtlasSuggestion) {
        let addressToken = place.addressTokens
                        var data: [String: Any?] = [
                            "distance": place.distance,
                            "eLoc": place.eLoc,
                            "email": place.email,
                            "entryLatitude": place.entryLatitude,
                            "entryLongitude": place.entryLongitude,
                            "keywords": place.keywords,
                            "landlineNo": place.landlineNo,
                            "latitude": place.latitude,
                            "longitude": place.longitude,
                            "mobileNo": place.mobileNo,
                            "orderIndex": place.orderIndex,
                            "placeAddress": place.placeAddress,
                            "placeName": place.placeName,
                            "type": place.type,
                            "categoryCode": place.categoryCode,
                            "hourOfOperation": place.hourOfOperation,
                            "richInfo": place.richInfoDictionary]
                        if addressToken != nil {
                            data["addressTokens"] = ["poi": addressToken?.poi,
                                               "city": addressToken?.city,
                                               "district":addressToken?.district,
                                               "houseName": addressToken?.houseName,
                                               "houseNumber": addressToken?.houseNumber,
                                               "locality": addressToken?.locality,
                                               "pincode": addressToken?.pincode,
                                               "state": addressToken?.state,
                                               "street": addressToken?.street,
                                               "subDistrict": addressToken?.subDistrict,
                                               "subLocality": addressToken?.subLocality,
                                               "subSubLocality": addressToken?.subSubLocality,
                                               "village": addressToken?.village]
                        
                        }
        let jsonData = try? JSONSerialization.data(withJSONObject: data, options: [])
        let jsonString = String(data: jsonData!, encoding: .utf8)
        result(jsonString ?? "nil")
        flutterViewController.dismiss(animated: false, completion: nil)
        
    }
    
}

extension SwiftMapmyindiaNearbyPlugin: MapmyIndiaNearbyCategoriesViewControllerDelegate {
    public func didSelectNearbyIcon(place: MapmyIndiaAtlasSuggestion) {
        self.sendCallback(place: place)
        
    }
    
    public func didSelectNearbySuggestionFromTable(place: MapmyIndiaAtlasSuggestion) {
        self.sendCallback(place: place)
    }
    
    public func didNextButtonClicked(refLocation: String?, selectedCategories: [MapmyIndiaNearbyCategories]?, error: String?) {}
    
    public func didRequestNearbyResult(result: NearbyResult?, error: NSError?) {}
    
    
}
